
# Getting Started with Account and Transaction API Specification

## Introduction

Swagger for Account and Transaction API Specification. This is heavily derived from the Open Banking UK API - see www.openbanking.org.uk for details.

## Install the Package

The package is compatible with Python versions `3.7+`.
Install the package from PyPi using the following pip command:

```bash
pip install pnz-payments-sdk==1.0.0
```

You can also view the package at:
https://pypi.python.org/pypi/pnz-payments-sdk/1.0.0

## Test the SDK

You can test the generated SDK and the server with test cases. `unittest` is used as the testing framework and `pytest` is used as the test runner. You can run the tests as follows:

Navigate to the root directory of the SDK and run the following commands

```
pip install -r test-requirements.txt
pytest
```

## Initialize the API Client

**_Note:_** Documentation for the client can be found [here.](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/client.md)

The following parameters are configurable for the API Client:

| Parameter | Type | Description |
|  --- | --- | --- |
| `environment` | `Environment` | The API environment. <br> **Default: `Environment.PRODUCTION`** |
| `http_client_instance` | `HttpClient` | The Http Client passed from the sdk user for making requests |
| `override_http_client_configuration` | `bool` | The value which determines to override properties of the passed Http Client from the sdk user |
| `http_call_back` | `HttpCallBack` | The callback value that is invoked before and after an HTTP call is made to an endpoint |
| `timeout` | `float` | The value to use for connection timeout. <br> **Default: 60** |
| `max_retries` | `int` | The number of times to retry an endpoint call if it fails. <br> **Default: 0** |
| `backoff_factor` | `float` | A backoff factor to apply between attempts after the second try. <br> **Default: 2** |
| `retry_statuses` | `Array of int` | The http statuses on which retry is to be done. <br> **Default: [408, 413, 429, 500, 502, 503, 504, 521, 522, 524]** |
| `retry_methods` | `Array of string` | The http methods on which retry is to be done. <br> **Default: ['GET', 'PUT']** |
| `bearer_auth_credentials` | [`BearerAuthCredentials`](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/auth/oauth-2-bearer-token.md) | The credential object for OAuth 2 Bearer token |

The API client can be initialized as follows:

```python
client = AccountandtransactionapispecificationClient(
    bearer_auth_credentials=BearerAuthCredentials(
        access_token='AccessToken'
    ),
    environment=Environment.PRODUCTION
)
```

## Environments

The SDK can be configured to use a different environment for making API calls. Available environments are:

### Fields

| Name | Description |
|  --- | --- |
| production | **Default** |
| environment2 | - |

## Authorization

This API uses the following authentication schemes.

* [`HTTPBearerAuthentication (OAuth 2 Bearer token)`](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/auth/oauth-2-bearer-token.md)

## List of APIs

* [Direct Debits](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/direct-debits.md)
* [Scheduled Payments](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/scheduled-payments.md)
* [Standing Orders](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/standing-orders.md)
* [Accounts](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/accounts.md)
* [Balances](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/balances.md)
* [Beneficiaries](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/beneficiaries.md)
* [Offers](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/offers.md)
* [Party](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/party.md)
* [Statements](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/statements.md)
* [Transactions](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/controllers/transactions.md)

## Classes Documentation

* [Utility Classes](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/utility-classes.md)
* [HttpResponse](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/http-response.md)
* [HttpRequest](https://www.github.com/sdks-io/pnz-payments-python-sdk/tree/1.0.0/doc/http-request.md)

